@@ -1,14 +1,14 @@
-from __future__ import print_function
 import datetime
-import warnings
+import os
 import re
+import time
+import warnings
 
 from .config import Configuration
-from .utils import trace, string_types, utc
+from .config import Version as PkgVersion
+from .utils import iter_entry_points
+from .utils import trace
 
-from pkg_resources import iter_entry_points
-
-from pkg_resources import parse_version as pkg_parse_version
 
 SEMVER_MINOR = 2
 SEMVER_PATCH = 3
@@ -16,7 +16,7 @@ SEMVER_LEN = 3
 
 
 def _parse_version_tag(tag, config):
-    tagstring = tag if not isinstance(tag, string_types) else str(tag)
+    tagstring = tag if not isinstance(tag, str) else str(tag)
     match = config.tag_regex.match(tagstring)
 
     result = None
@@ -32,34 +32,10 @@ def _parse_version_tag(tag, config):
             "suffix": match.group(0)[match.end(key) :],
         }
 
-    trace("tag '{}' parsed to {}".format(tag, result))
+    trace(f"tag '{tag}' parsed to {result}")
     return result
 
 
-def _get_version_class():
-    modern_version = pkg_parse_version("1.0")
-    if isinstance(modern_version, tuple):
-        return None
-    else:
-        return type(modern_version)
-
-
-VERSION_CLASS = _get_version_class()
-
-
-class SetuptoolsOutdatedWarning(Warning):
-    pass
-
-
-# append so integrators can disable the warning
-warnings.simplefilter("error", SetuptoolsOutdatedWarning, append=True)
-
-
-def _warn_if_setuptools_outdated():
-    if VERSION_CLASS is None:
-        warnings.warn("your setuptools is too old (<12)", SetuptoolsOutdatedWarning)
-
-
 def callable_or_entrypoint(group, callable_or_name):
     trace("ep", (group, callable_or_name))
 
@@ -83,7 +59,7 @@ def tag_to_version(tag, config=None):
 
     tagdict = _parse_version_tag(tag, config)
     if not isinstance(tagdict, dict) or not tagdict.get("version", None):
-        warnings.warn("tag {!r} no version found".format(tag))
+        warnings.warn(f"tag {tag!r} no version found")
         return None
 
     version = tagdict["version"]
@@ -96,9 +72,8 @@ def tag_to_version(tag, config=None):
             )
         )
 
-    if VERSION_CLASS is not None:
-        version = pkg_parse_version(version)
-        trace("version", repr(version))
+    version = config.version_cls(version)
+    trace("version", repr(version))
 
     return version
 
@@ -117,7 +92,7 @@ def tags_to_versions(tags, config=None):
     return result
 
 
-class ScmVersion(object):
+class ScmVersion:
     def __init__(
         self,
         tag_version,
@@ -127,7 +102,8 @@ class ScmVersion(object):
         preformatted=False,
         branch=None,
         config=None,
-        **kw
+        node_date=None,
+        **kw,
     ):
         if kw:
             trace("unknown args", kw)
@@ -136,7 +112,10 @@ class ScmVersion(object):
             distance = 0
         self.distance = distance
         self.node = node
-        self.time = datetime.datetime.now(utc)
+        self.node_date = node_date
+        self.time = datetime.datetime.utcfromtimestamp(
+            int(os.environ.get("SOURCE_DATE_EPOCH", time.time()))
+        )
         self._extra = kw
         self.dirty = dirty
         self.preformatted = preformatted
@@ -169,7 +148,8 @@ class ScmVersion(object):
             node=self.node,
             dirty=self.dirty,
             branch=self.branch,
-            **kw
+            node_date=self.node_date,
+            **kw,
         )
 
     def format_choice(self, clean_format, dirty_format, **kw):
@@ -183,7 +163,7 @@ class ScmVersion(object):
 def _parse_tag(tag, preformatted, config):
     if preformatted:
         return tag
-    if VERSION_CLASS is None or not isinstance(tag, VERSION_CLASS):
+    if not isinstance(tag, config.version_cls):
         tag = tag_to_version(tag, config)
     return tag
 
@@ -196,7 +176,7 @@ def meta(
     preformatted=False,
     branch=None,
     config=None,
-    **kw
+    **kw,
 ):
     if not config:
         warnings.warn(
@@ -205,7 +185,7 @@ def meta(
         )
     parsed_version = _parse_tag(tag, preformatted, config)
     trace("version", tag, "->", parsed_version)
-    assert parsed_version is not None, "cant parse version %s" % tag
+    assert parsed_version is not None, "Can't parse version %s" % tag
     return ScmVersion(
         parsed_version, distance, node, dirty, preformatted, branch, config, **kw
     )
@@ -226,13 +206,26 @@ def _bump_dev(version):
         return
 
     prefix, tail = version.rsplit(".dev", 1)
-    assert tail == "0", "own dev numbers are unsupported"
+    if tail != "0":
+        raise ValueError(
+            "choosing custom numbers for the `.devX` distance "
+            "is not supported.\n "
+            "The {version} can't be bumped\n"
+            "Please drop the tag or create a new supported one".format(version=version)
+        )
     return prefix
 
 
 def _bump_regex(version):
-    prefix, tail = re.match(r"(.*?)(\d+)$", version).groups()
-    return "%s%d" % (prefix, int(tail) + 1)
+    match = re.match(r"(.*?)(\d+)$", version)
+    if match is None:
+        raise ValueError(
+            "{version} does not end with a number to bump, "
+            "please correct or use a custom version scheme".format(version=version)
+        )
+    else:
+        prefix, tail = match.groups()
+        return "%s%d" % (prefix, int(tail) + 1)
 
 
 def guess_next_dev_version(version):
@@ -243,7 +236,10 @@ def guess_next_dev_version(version):
 
 
 def guess_next_simple_semver(version, retain, increment=True):
-    parts = [int(i) for i in str(version).split(".")[:retain]]
+    try:
+        parts = [int(i) for i in str(version).split(".")[:retain]]
+    except ValueError:
+        raise ValueError(f"{version} can't be parsed as numeric version")
     while len(parts) < retain:
         parts.append(0)
     if increment:
@@ -267,18 +263,22 @@ def simplified_semver_version(version):
             )
 
 
-def release_branch_semver(version):
+def release_branch_semver_version(version):
     if version.exact:
         return version.format_with("{tag}")
     if version.branch is not None:
         # Does the branch name (stripped of namespace) parse as a version?
         branch_ver = _parse_version_tag(version.branch.split("/")[-1], version.config)
         if branch_ver is not None:
+            branch_ver = branch_ver["version"]
+            if branch_ver[0] == "v":
+                # Allow branches that start with 'v', similar to Version.
+                branch_ver = branch_ver[1:]
             # Does the branch version up to the minor part match the tag? If not it
             # might be like, an issue number or something and not a version number, so
             # we only want to use it if it matches.
             tag_ver_up_to_minor = str(version.tag).split(".")[:SEMVER_MINOR]
-            branch_ver_up_to_minor = branch_ver["version"].split(".")[:SEMVER_MINOR]
+            branch_ver_up_to_minor = branch_ver.split(".")[:SEMVER_MINOR]
             if branch_ver_up_to_minor == tag_ver_up_to_minor:
                 # We're in a release/maintenance branch, next is a patch/rc/beta bump:
                 return version.format_next_version(guess_next_version)
@@ -286,6 +286,99 @@ def release_branch_semver(version):
     return version.format_next_version(guess_next_simple_semver, retain=SEMVER_MINOR)
 
 
+def release_branch_semver(version):
+    warnings.warn(
+        "release_branch_semver is deprecated and will be removed in future. "
+        + "Use release_branch_semver_version instead",
+        category=DeprecationWarning,
+        stacklevel=2,
+    )
+    return release_branch_semver_version(version)
+
+
+def no_guess_dev_version(version):
+    if version.exact:
+        return version.format_with("{tag}")
+    else:
+        return version.format_with("{tag}.post1.dev{distance}")
+
+
+def date_ver_match(ver):
+    match = re.match(
+        (
+            r"^(?P<date>(?P<year>\d{2}|\d{4})(?:\.\d{1,2}){2})"
+            r"(?:\.(?P<patch>\d*)){0,1}?$"
+        ),
+        str(ver),
+    )
+    return match
+
+
+def guess_next_date_ver(version, node_date=None, date_fmt=None, version_cls=None):
+    """
+    same-day -> patch +1
+    other-day -> today
+
+    distance is always added as .devX
+    """
+    match = date_ver_match(version)
+    if match is None:
+        warnings.warn(
+            f"{version} does not correspond to a valid versioning date, "
+            "assuming legacy version"
+        )
+        if date_fmt is None:
+            date_fmt = "%y.%m.%d"
+
+    # deduct date format if not provided
+    if date_fmt is None:
+        date_fmt = "%Y.%m.%d" if len(match.group("year")) == 4 else "%y.%m.%d"
+    head_date = node_date or datetime.date.today()
+    # compute patch
+    if match is None:
+        tag_date = datetime.date.today()
+    else:
+        tag_date = datetime.datetime.strptime(match.group("date"), date_fmt).date()
+    if tag_date == head_date:
+        patch = "0" if match is None else (match.group("patch") or "0")
+        patch = int(patch) + 1
+    else:
+        if tag_date > head_date and match is not None:
+            # warn on future times
+            warnings.warn(
+                "your previous tag  ({}) is ahead your node date ({})".format(
+                    tag_date, head_date
+                )
+            )
+        patch = 0
+    next_version = "{node_date:{date_fmt}}.{patch}".format(
+        node_date=head_date, date_fmt=date_fmt, patch=patch
+    )
+    # rely on the Version object to ensure consistency (e.g. remove leading 0s)
+    if version_cls is None:
+        version_cls = PkgVersion
+    next_version = str(version_cls(next_version))
+    return next_version
+
+
+def calver_by_date(version):
+    if version.exact and not version.dirty:
+        return version.format_with("{tag}")
+    # TODO: move the release-X check to a new scheme
+    if version.branch is not None and version.branch.startswith("release-"):
+        branch_ver = _parse_version_tag(version.branch.split("-")[-1], version.config)
+        if branch_ver is not None:
+            ver = branch_ver["version"]
+            match = date_ver_match(ver)
+            if match:
+                return ver
+    return version.format_next_version(
+        guess_next_date_ver,
+        node_date=version.node_date,
+        version_cls=version.config.version_cls,
+    )
+
+
 def _format_local_with_time(version, time_format):
 
     if version.exact or version.node is None:
@@ -321,19 +414,47 @@ def postrelease_version(version):
         return version.format_with("{tag}.post{distance}")
 
 
+def _get_ep(group, name):
+    for ep in iter_entry_points(group, name):
+        trace("ep found:", ep.name)
+        return ep.load()
+
+
+def _iter_version_schemes(entrypoint, scheme_value, _memo=None):
+    if _memo is None:
+        _memo = set()
+    if isinstance(scheme_value, str):
+        scheme_value = _get_ep(entrypoint, scheme_value)
+
+    if isinstance(scheme_value, (list, tuple)):
+        for variant in scheme_value:
+            if variant not in _memo:
+                _memo.add(variant)
+                yield from _iter_version_schemes(entrypoint, variant, _memo=_memo)
+    elif callable(scheme_value):
+        yield scheme_value
+
+
+def _call_version_scheme(version, entypoint, given_value, default):
+    for scheme in _iter_version_schemes(entypoint, given_value):
+        result = scheme(version)
+        if result is not None:
+            return result
+    return default
+
+
 def format_version(version, **config):
     trace("scm version", version)
     trace("config", config)
     if version.preformatted:
         return version.tag
-    version_scheme = callable_or_entrypoint(
-        "setuptools_scm.version_scheme", config["version_scheme"]
+    main_version = _call_version_scheme(
+        version, "setuptools_scm.version_scheme", config["version_scheme"], None
     )
-    local_scheme = callable_or_entrypoint(
-        "setuptools_scm.local_scheme", config["local_scheme"]
-    )
-    main_version = version_scheme(version)
     trace("version", main_version)
-    local_version = local_scheme(version)
+    assert main_version is not None
+    local_version = _call_version_scheme(
+        version, "setuptools_scm.local_scheme", config["local_scheme"], "+unknown"
+    )
     trace("local_version", local_version)
-    return version_scheme(version) + local_scheme(version)
+    return main_version + local_version
